@@ -11,6 +11,7 @@ import sys
 import json
 import shutil
 import platform
+from types import SimpleNamespace
 
 from subprocess import Popen, PIPE
 
@@ -45,6 +46,16 @@ if os.name == "nt":
 def in_rlang():
     return r2lang is not None and r2lang.cmd is not None
 
+def in_r2():
+    """Return wheter r2pipe is called from radare2 environment or the system shell
+        """
+    try:
+        import os
+        a = int(os.environ["R2PIPE_IN"])
+        b = int(os.environ["R2PIPE_OUT"])
+        return a > 0 and b > 0
+    except (ValueError, KeyError, TypeError):
+        return False
 
 def jo2po(jo):
     from collections import namedtuple
@@ -88,8 +99,6 @@ def get_radare_path():
 class OpenBase(object):
     """Class representing an r2pipe connection with a running radare2 instance
         Class body derived from __init__.py "open" class.
-    
-    
         """
 
     def __init__(self, filename="", flags=[]):
@@ -111,9 +120,12 @@ class OpenBase(object):
         self.use_cache = False
         self.cache = {}
         self.asyn = False
-        if not filename and in_rlang():
-            self._cmd = self._cmd_rlang
-            return
+        if not filename:
+            if in_rlang():
+                self._cmd = self._cmd_rlang
+                return
+            if not in_r2():
+                raise EnvironmentError("No R2PIPE_IN or R2PIPE_OUT defined")
         try:
             if os.name == "nt":
                 # if r2pipe_path environment is not set then fail below
@@ -148,10 +160,10 @@ class OpenBase(object):
             self._cmd = self._cmd_pipe
             self.url = "#!pipe"
             return
-        except:
+        except (KeyError, ValueError, OSError, NameError) as e:
             pass
         if filename.startswith("#!pipe"):
-            raise Exception("ERROR: Cannot use #!pipe without R2PIPE_{IN|OUT} env")
+            raise EnvironmentError("ERROR: Cannot use #!pipe without R2PIPE_{IN|OUT} env")
 
     def invalidate_cache(self):
         self.cache = {}
@@ -196,7 +208,7 @@ class OpenBase(object):
             return self.cache[cmd]
         cmd = cmd.strip().replace("\n", ";")
         if not has_native:
-            raise Exception("No native ctypes connector available")
+            raise ImportError("No native ctypes connector available")
         if not hasattr(self, "native"):
             self.native = RCore()
             self.native.cmd_str("o " + self.uri)
@@ -235,7 +247,21 @@ class OpenBase(object):
                 import asyncio
                 asyncio.get_event_loop().run_until_complete(asyncio.sleep(0.1))
 
+    # r2pipe2
+    def cmd2(self, cmd, **kwargs):
+        s = { "cmd": cmd }
+        r = self.call(json.dumps(s))
+        return json.loads(r, object_hook=lambda d: SimpleNamespace(**d))
+
+    def cmd2j(self, cmd, **kwargs):
+        s = { "cmd": cmd, "trim": True, "json": True }
+        r = self.call(json.dumps(s))
+        return json.loads(r, object_hook=lambda d: SimpleNamespace(**d))
+
     # r2 commands
+    def call(self, cmd, **kwargs):
+        return self.cmd("'" + cmd)
+
     def cmd(self, cmd, **kwargs):
         """Run an r2 command return string with result
                 Args:
@@ -267,7 +293,7 @@ class OpenBase(object):
         try:
             data = json.loads(result)
         except (ValueError, KeyError, TypeError) as e:
-            sys.stderr.write("r2pipe.cmdj.Error: %s\n" % (e))
+            sys.stderr.write(f"r2pipe.cmdj.Error: {e}\n")
             data = None
         return data
 
@@ -282,7 +308,7 @@ class OpenBase(object):
         try:
             return jo2po(result)
         except (ValueError, KeyError, TypeError) as e:
-            sys.stderr.write("r2pipe.cmdj.Error: %s\n" % (e))
+            sys.stderr.write(f"r2pipe.cmdj.Error: {e}\n")
         return None
 
     def syscmd(self, cmd):
@@ -306,6 +332,6 @@ class OpenBase(object):
         try:
             data = json.loads(self.syscmd(cmd))
         except (ValueError, KeyError, TypeError) as e:
-            sys.stderr.write("r2pipe.syscmdj.Error %s\n" % (e))
+            sys.stderr.write(f"r2pipe.syscmdj.Error {e}\n")
             data = None
         return data
