@@ -20,24 +20,30 @@ Example:
   > import r2pipe
   > r = r2pipe.open("/bin/ls")
   > print(r.cmd("pd 10"))
-  > print(r.cmdj("aoj")[0]['size'])
-  > r.quit()
+  > print(r.cmdj("aoj")[0]['size']) > r.quit()
 """
 
-import os
-import sys
-import time
-from r2pipe.open_sync import open as r2pipe_open
-
-open = r2pipe_open
+from r2pipe.open_sync import open
 
 try:
     import r2lang
 except ImportError:
     r2lang = None
 
-VERSION = "1.8.8"
+VERSION = "1.9.6"
 
+def in_r2():
+    """Return wheter r2pipe is called from radare2 environment or the system shell
+        """
+    try:
+        import os
+        a = int(os.environ["R2PIPE_IN"])
+        b = int(os.environ["R2PIPE_OUT"])
+        return a > 0 and b > 0
+    except (ValueError, KeyError, TypeError):
+        return False
+
+from r2pipe.open_base import in_rlang
 
 def version():
     """Return string with the version of the r2pipe library
@@ -47,10 +53,14 @@ def version():
 
 # Open class is now in open_base.py
 if __name__ == "__main__":
+    import os
+    import sys
+    from r2pipe.open_sync import open as r2pipe_open
     print("[+] Spawning r2 tcp and http servers")
     os.system("pkill r2")
     os.system("radare2 -qc.:9080 /bin/ls &")
     os.system("radare2 -qc=h /bin/ls &")
+    import time
     time.sleep(1)
 
     if sys.version_info <= (3, 0):
