@@ -13,13 +13,13 @@ from subprocess import Popen, PIPE
 from r2pipe.open_base import OpenBase
 
 def no_urlopen():
-  raise IOError
+    raise IOError
 try:
-  from urllib.error import URLError
-  from urllib.request import urlopen
-except:
-  URLError = IOError
-  urlopen = no_urlopen
+    from urllib.error import URLError
+    from urllib.request import urlopen
+except ImportError:
+    URLError = IOError
+    urlopen = no_urlopen
 
 
 try:
@@ -27,7 +27,6 @@ try:
 except ImportError:
     fcntl = None
 
-
 class open(OpenBase):
     def __enter__(self):
         return
@@ -46,7 +45,7 @@ class open(OpenBase):
         elif filename.startswith("tcp://"):
             r = re.match(r"tcp://(\d+\.\d+.\d+.\d+):(\d+)/?", filename)
             if not r:
-                raise Exception("String doesn't match tcp format")
+                raise ValueError("String doesn't match tcp format")
             self._cmd = self._cmd_tcp
             self.conn = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
             self.conn.connect((r.group(1), int(r.group(2))))
@@ -54,7 +53,7 @@ class open(OpenBase):
             self._cmd = self._cmd_process
             if radare2home is not None:
                 if not os.path.isdir(radare2home):
-                    raise Exception(
+                    raise ValueError(
                         "`radare2home` passed is invalid, leave it None or put a valid path to r2 folder"
                     )
                 r2e = os.path.join(radare2home, "radare2")
@@ -72,8 +71,8 @@ class open(OpenBase):
                 self.process = Popen(
                     cmd, shell=False, stdin=PIPE, stdout=PIPE, bufsize=0
                 )
-            except:
-                raise Exception("ERROR: Cannot find radare2 in PATH")
+            except (OSError, FileNotFoundError) as e:
+                raise FileNotFoundError("ERROR: Cannot find radare2 in PATH") from e
 
             if os.name == "nt":
                 # On windows-spawn method we need to read the null byte twice
@@ -88,8 +87,8 @@ class open(OpenBase):
                         ch = r.read(1)
                         if ch == b'\x00':
                             break
-                except:
-                    raise Exception("ERROR: Cannot open %s" % filename)
+                except (IOError, OSError) as e:
+                    raise IOError(f"ERROR: Cannot open {filename}") from e
 
     @staticmethod
     def __make_non_blocking(fd):
@@ -107,7 +106,7 @@ class open(OpenBase):
 
         try:
             from ctypes import POINTER
-        except:
+        except ImportError:
             from ctypes.wintypes import POINTER
 
         LPDWORD = POINTER(DWORD)
@@ -122,50 +121,75 @@ class open(OpenBase):
         return res != 0
 
     def _cmd_process(self, cmd):
-        cmd = cmd.strip().replace("\n", ";")
-        try:
-            self.process.stdin.write((cmd + "\n").encode("utf8"))
-        except:
-            return ''
-        r = self.process.stdout
-        self.process.stdin.flush()
-        out = bytearray()
-        foo = None
-        while True:
-            if self.process.poll() is not None:
-                raise RuntimeError(f"Process terminated unexpectedly trying to run the command {cmd}\n{self.process}")
+        # Add a simple mutex-like lock mechanism using threading
+        import threading
+        if not hasattr(self, '_cmd_lock'):
+            self._cmd_lock = threading.Lock()
+        
+        # Acquire lock to ensure commands don't interfere with each other
+        with self._cmd_lock:
+            # Ensure pending buffer is cleared before starting a new command to avoid mixing
+            old_pending = self.pending
+            self.pending = b""
+            
+            cmd = cmd.strip().replace("\n", ";")
             try:
-                null_start = False
-                if len(self.pending) > 0:
-                    foo = self.pending
-                    self.pending = b""
-                else:
-                    foo = r.read(4096)
-                    if os.name == "nt":
-                        if foo.startswith(b"\x00"):
-                            foo = foo[1:]
-                            null_start = True
-                if foo:
-                    zro = foo.find(b"\x00")
-                    if zro != -1:
-                        out += foo[0:zro]
-                        if zro  < len(foo):
-                            self.pending = foo[zro + 1:]
-                        break
-                    out += foo
-                elif null_start:
-                    break
-
-            except KeyboardInterrupt as e:
-                raise e
+                self.process.stdin.write((cmd + "\n").encode("utf8"))
             except:
-                pass
-        return out.decode("utf-8", errors="ignore")
+                self.pending = old_pending  # Restore pending on failure
+                return ''
+                
+            r = self.process.stdout
+            self.process.stdin.flush()
+            out = bytearray()
+            foo = None
+            
+            # First read any pending data from previous commands if exists
+            if old_pending:
+                zro = old_pending.find(b"\x00")
+                if zro != -1:
+                    # This is a complete response from previous command, process it first
+                    out += old_pending[0:zro]
+                    if zro + 1 < len(old_pending):
+                        self.pending = old_pending[zro + 1:]
+                    return out.decode("utf-8", errors="ignore")
+            
+            # Main read loop for current command
+            while True:
+                if self.process.poll() is not None:
+                    raise RuntimeError(f"Process terminated unexpectedly trying to run the command {cmd}\n{self.process}")
+                try:
+                    null_start = False
+                    if len(self.pending) > 0:
+                        foo = self.pending
+                        self.pending = b""
+                    else:
+                        foo = r.read(4096)
+                        if os.name == "nt":
+                            if foo.startswith(b"\x00"):
+                                foo = foo[1:]
+                                null_start = True
+                    if foo:
+                        zro = foo.find(b"\x00")
+                        if zro != -1:
+                            out += foo[0:zro]
+                            if zro + 1 < len(foo):
+                                self.pending = foo[zro + 1:]
+                            break
+                        out += foo
+                    elif null_start:
+                        break
+
+                except KeyboardInterrupt as e:
+                    raise e
+                except:
+                    pass
+            return out.decode("utf-8", errors="ignore")
 
     def _cmd_http(self, cmd):
         try:
             quocmd = urllib.parse.quote(cmd)
-            response = urlopen("{uri}/{cmd}".format(uri=self.uri, cmd=quocmd))
+            response = urlopen(f"{self.uri}/{quocmd}")
             return response.read().decode("utf-8", errors="ignore")
         except URLError:
             pass
